// Implementazione del Client RMI

import java.rmi.*;
import java.io.*;
import java.net.*;

class Client {

  // Avvio del Client RMI
  public static void main(String[] args) {
    final int REGISTRYPORT = 1099;
    String registryHost = "localhost";
    String serviceName = "ServerImpl";
    int port = REGISTRYPORT;
    Socket sock = null;
    DataInputStream in = null;
    BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));

    // Controllo dei parametri della riga di comando
    if(args.length == 1){
      try{
        port = Integer.parseInt(args[0]);
      } catch(NumberFormatException e){
        System.out.println("Not a valid port. Port set to default.");
      }
    }

    // Connessione al servizio RMI remoto
    try{
      String completeName = "//"+registryHost+":"+port+"/"+serviceName;
      RemOp serverRMI = (RemOp) Naming.lookup(completeName);
      System.out.println("ClientRMI: Servizio \""+serviceName+"\" connesso");
      System.out.println("\nRichieste di servizio fino a fine file");
      String service;
      System.out.println("G: get con client attivo, g: get con client passivo, EOF: termina");
      while ((service = stdIn.readLine()) != null){
        if(service.length() == 0){
          System.out.println("G: get con client attivo, g: get con client passivo, EOF: termina");
          continue;
        }
        if(service.charAt(0) == 'G'){
          // trasferimento con client attivo
          System.out.println("Immetti il nome del direttorio che vuoi trasferire");
          String nomeDir;
          if((nomeDir = stdIn.readLine()) != null){
            StructEndpointList struttura;
            try{
              struttura = serverRMI.activeClientGet(nomeDir);
            }catch(RemoteException e){
              System.out.println("Errore: "+e.getMessage());
              continue;
            } //try-catch
            String[] listaNomi = struttura.listaNomi;
            int[] listaDimensioni = struttura.listaDimensioni;
            if(listaNomi.length != listaDimensioni.length){
              System.out.println("Errore: le dimensioni non corrispondono ai nomi dei file");
              continue;
            }
            
            /** Apertura connessione (connect) con l'endpoint specificato nella struttura **/
            try {
              sock = new Socket(struttura.serverAddr,struttura.serverPort);
              in = new DataInputStream(sock.getInputStream());
              for(int i=0;i<listaNomi.length;i++){
                DataOutputStream fOut = new DataOutputStream(new FileOutputStream(listaNomi[i]));
                // ricezione dell'i-esimo file
                for(int k=0;k<listaDimensioni[i];k++) fOut.write(in.read());
              } //for-ricezione file
              sock.close();
            } catch (IOException ioex) {
              System.out.println("Errore nella creazione socket (client attivo), termino");
              ioex.printStackTrace();
              System.exit(4);
            } //try-catch
          }else{
            System.out.println("Nome file non valido");
            continue;
          } //if-else nomeFile
        } else if(service.charAt(0) == 'g'){
          //trasferimento con server attivo
          // creazione socket
          ServerSocket servSock = null;
          try {
            servSock = new ServerSocket(0);
            servSock.setReuseAddress(true);
          } catch (IOException ioex) {
            System.out.println("Errore nella creazione socket (server attivo), termino");
            ioex.printStackTrace();
            System.exit(5);
          } //try-catch
          System.out.println("Immetti il nome del direttorio che vuoi trasferire");
          String nomeDir;
          if((nomeDir = stdIn.readLine()) != null){
            StructList struttura;
            try{
              struttura = serverRMI.activeServerGet(nomeDir, servSock.getInetAddress(), servSock.getLocalPort());
            }catch(RemoteException e){
              System.out.println("Errore: "+e.getMessage());
              continue;
            } //try-catch
            String[] listaNomi = struttura.listaNomi;
            int[] listaDimensioni = struttura.listaDimensioni;
            if(listaNomi.length != listaDimensioni.length){
              System.out.println("Errore: le dimensioni non corrispondono ai nomi dei file");
              continue;
            }
            try{
              sock = servSock.accept();
              in = new DataInputStream(sock.getInputStream());
              for(int i=0;i<listaNomi.length;i++){
                DataOutputStream fOut = new DataOutputStream(new FileOutputStream(listaNomi[i]));
                // ricezione dell'i-esimo file
                for(int k=0;k<listaDimensioni[i];k++) fOut.write(in.read());
              } //for-ricezione file
              sock.close();
              servSock.close();
            }catch(IOException ioex){
              System.out.println("Errore socket (server attivo), termino");
              ioex.printStackTrace();
              System.exit(6);
            } //try-catch
          }else{
            System.out.println("Nome direttorio non valido");
            continue;
          } //if-else nomeDir
        } //if-else operazione
      System.out.println("G: get con client attivo, g: get con client passivo, EOF: termina");    
      } // while
    }catch(Exception e){
      System.err.println("ClientRMI: " + e.getMessage());
      e.printStackTrace();
      System.exit(1);
    } // try-catch
  } // main
} // class
