// Implementazione del Server RMI

import java.rmi.*;
import java.rmi.server.*;
import java.io.*;
import java.net.*;

public class ServerImpl extends UnicastRemoteObject implements RemOp {
  final static int REGISTRYPORT = 1099;
  private int transferPort = REGISTRYPORT+1;
  
  // Costruttore
  public ServerImpl() throws RemoteException {
    super();
  }

  // Implementazione metodi astratti
  public synchronized StructEndpointList activeClientGet(String dirName) throws RemoteException{
    
    InetAddress transferAddr;
    try{
      transferAddr = InetAddress.getLocalHost();
    }catch(java.net.UnknownHostException e){
      System.out.println("Errore in lettura host");
      throw new RemoteException("Host sconosciuto");
    }
    
    /** Thread di gestione del trasferimento di dirName usando la porta transferPort **/
    Sender server = new Sender(dirName, transferPort);
    server.start();
    
    // ritorno della struttura contenente endpoint per il trasferimento e dell'elenco file
    String[] nomiFile = null;
    int[] dimensioniFile = null;
    File d = new File(dirName);
    if(!d.isDirectory()) throw new RemoteException("No Directory '"+dirName+"' Found");
    int n = d.listFiles().length;
    nomiFile = new String[n];
    dimensioniFile = new int[n];
    for (int i = 0; i < n; i++){
      nomiFile[i] = d.listFiles()[i].getName();
      dimensioniFile[i] = (int)d.listFiles()[i].length();
    }
    StructEndpointList struttura = new StructEndpointList(transferAddr,transferPort,nomiFile,dimensioniFile);
    transferPort++;
    return struttura;
  } //activeClientGet
  
  public synchronized StructList activeServerGet(String dirName, InetAddress clientAddr, int clientPort) throws RemoteException{
    
    // preparo la struttura contenente l'elenco file
    String[] nomiFile = null;
    int[] dimensioniFile = null;
    File d = new File(dirName);
    if(!d.isDirectory()) throw new RemoteException("No Directory '"+dirName+"' Found");
    int n = d.listFiles().length;
    nomiFile = new String[n];
    dimensioniFile = new int[n];
    for (int i = 0; i < n; i++){
      nomiFile[i] = d.listFiles()[i].getName();
      dimensioniFile[i] = (int)d.listFiles()[i].length();
    }
    StructList struttura = new StructList(nomiFile, dimensioniFile);
    transferPort++; 
    
    /** Thread di gestione del trasferimento di dirName verso l'endpoint del client usando la porta transferPort **/
    //thread(dirName,port,clientAddress,clientPort)
    Sender server = new Sender(dirName, transferPort, clientAddr, clientPort);
    server.start();
    
    
    return struttura;
  } //activeServerGet

  // Avvio del Server RMI
  public static void main(String[] args) {

    String registryHost = "localhost";
    String serviceName = "ServerImpl";
    int port = REGISTRYPORT;
    if(args.length == 1){
      try{
        port = Integer.parseInt(args[1]);
      } catch(NumberFormatException e){
        System.out.println("Not a valid port.");
      } //try-catch
    }
    
    // Registrazione del servizio RMI
    try{
      String completeName = "//"+registryHost+":"+port+"/"+serviceName;
      ServerImpl serverRMI = new ServerImpl();
      Naming.rebind(completeName, serverRMI);
      System.out.println("Server RMI: Servizio \""+serviceName+"\" registrato");
    } catch(Exception e){
      System.err.println("Server RMI \""+serviceName+"\": "+ e.getMessage());
      e.printStackTrace();
      System.exit(1);
    } //try-catch
  } //main
} //class
